@@ -49,7 +49,7 @@ def register_action(
     id_or_action: str,
     title: str,
     *,
-    callback: CommandCallable,
+    callback: Callable[..., Any],
     category: str | None = ...,
     tooltip: str | None = ...,
     icon: IconOrDict | None = ...,
@@ -65,7 +65,7 @@ def register_action(
     id_or_action: str | Action,
     title: str | None = None,
     *,
-    callback: CommandCallable | None = None,
+    callback: Callable[..., Any] | None = None,
     category: str | None = None,
     tooltip: str | None = None,
     icon: IconOrDict | None = None,
@@ -231,7 +231,9 @@ def register_action(
             menus=menus,
             keybindings=keybindings,
         )
-    raise TypeError("'id_or_action' must be a string or an Action")
+    raise TypeError(
+        f"'id_or_action' must be a string or an Action not {type(id_or_action)}"
+    )
 
 
 def _register_action_str(
@@ -269,19 +271,4 @@ def _register_action_obj(app: Application | str, action: Action) -> DisposeCalla
     from app_model._app import Application
 
     app = app if isinstance(app, Application) else Application.get_or_create(app)
-
-    # commands
-    disposers = [app.commands.register_action(action)]
-    # menus
-    if dm := app.menus.append_action_menus(action):
-        disposers.append(dm)
-    # keybindings
-    if dk := app.keybindings.register_action_keybindings(action):
-        disposers.append(dk)
-
-    def _dispose() -> None:
-        for d in disposers:
-            d()
-
-    app._disposers.append((action.id, _dispose))
-    return _dispose
+    return app._register_action_obj(action)
